
require('tamejs').register();

var fs = require('fs');
var url = require('url');
var child_process = require('child_process');
var mustache = require('mustache');

exports.mixin = function(_){

    _.mixin({
        render : function(template, hash){
            return(mustache.render(template, hash))
        },
        wget : function(url, destPath, callback){
            var wget = 'wget -O ' + destPath + ' ' + url;
            // excute wget using child_process' exec function

            child_process.exec(wget, function(err, stdout, stderr) {
                if (err){ callback(err); }
                else{ callback(null); }
            });
        },
        dateHash : function(d){ 
            return({ 
                Type : 'Date',
            Year : d.getUTCFullYear(),
            Month : d.getUTCMonth(),
            Day : d.getUTCDate(),
            WeekDay : d.getUTCDay()
            });
        },
        dateTimeHash : function(d){
            return({ 
                Type : 'DateTime',
            Year : d.getUTCFullYear(),
            Month : d.getUTCMonth(),
            Day : d.getUTCDate(),
            WeekDay : d.getUTCDay(),
            Hour : d.getUTCHours(),
            Minute : d.getUTCMinutes(),
            Second : d.getUTCSeconds(),
            Millisecond : d.getUTCMilliseconds()
            });
        },
        request : require('superagent')
    });

    // this order matters
    _.diff = require('./diff.js').diffString;
    _.timeout = require('./timeout.js').timeout;
    _.fs = require('./fs.tjs').fs(_);
    _.code = require('./code.tjs').code(_);
    _.middleware = require('./middleware.js').middleware(_);
    _.http = require('./http.js').http(_);
    _.dry = require('./dry.js').mixin(_);
    require('./dry.server.js').mixin(_);
    _.getFirst = function(obj){
       for(var key in obj){
            if(obj.hasOwnProperty(key)){
                return(obj[key]);
            }
        }
        return(null);
    };
    _.getFirstExport = function(filePath){
        if(_.fs.Exists(filePath)){
            try{
                var exportsFile = require(filePath);
            }catch(e){
                console.log(filePath);
                throw(e);
            }
            return(_.getFirst(exportsFile));
        }
        return(null);
    };
    _.makeLogger = function(defaultLevel, dbName){
        var winston = require('winston');    
        defaultLevel = defaultLevel || 'debug';
        
        var options = { level : defaultLevel};
      
        if(dbName){
            var mongoDBTransport = new (require('winston-mongodb').MongoDB)({level : defaultLevel, db : dbName, keepAlive: 100});
            options.transports = [ new (winston.transports.Console)({level : defaultLevel}),
                                   mongoDBTransport
                                  ];
        }else{
            options.transports = [ new (winston.transports.Console)({level : defaultLevel})];
        }
        
        var logger = new (winston.Logger)(options);
        
        logger.setLevels(winston.config.syslog.levels);

        return(logger);
    };

};


